@@ -89,22 +89,20 @@ class PyScanner(MF):
     def __init__(self, path_filter, *args, **kwargs):
         self.path_filter = path_filter
         MF.__init__(self, *args, **kwargs)
-        self.imports = set()
 
     def scan(self, name, *arg, **karg):
-        self.name = name 
-        self.imports = set()
+        self.name = name
         return MF.run_script(self, name, *arg, **karg) 
 
     def import_hook(self, name, caller, *arg, **karg):
-        if self.path_filter(caller.__file__): 
-            self.imports.add(caller.__file__)
+        if caller.__file__ == self.name:# and self.path_filter(caller.__file__): 
             return MF.import_hook(self, name, caller, *arg, **karg) 
-        else: return None
 
     def __call__(self, node, env, path):
+        if str(node).split('.')[-1] not in ['py', 'pyc', 'pyo']: 
+            return []
         self.scan(str(node))
-        return list(self.imports)
+        return list(m.__file__ for m in self.modules.values() if self.path_filter(m.__file__))
 
 def ToList(input):
     if input is None: input = [] 
@@ -122,11 +120,12 @@ def _PyScript(env, target, source, command = "python $SOURCES",
     command = ToList(command)
 
     for n,s in enumerate(source): 
+        s = str(s)
         if s.strip()[:3] == "-m ":
             module = s.strip()[3:]
             s = os.popen("python -c 'import %s as M; print M.__file__'" 
                                     % module).read().strip()
-        if s[-3:] == "pyc": s = s[:-1]
+        if s[-3:] in ["pyc", "pyo"]: s = s[:-1]
         source[n] = s
     
     if capture in ["stdout", "out", "output"]: command[-1] += " > $TARGET"
@@ -150,6 +149,6 @@ class PYTOOL:
 
     def __call__(self, env):
         env.Append(SCANNERS = Scanner(function = PyScanner(self.path_filter), 
-                    skeys = ['.py']), recursive=self.recursive)
+                    name = "PyScanner", skeys = ['.py'], recursive=self.recursive))
         env.AddMethod(_PyScript, "PyScript")
         return env
